package psplot;

import java.awt.*;
import java.applet.*;

import real.*;
import psplot.*;

/* There is exactly one abstract method here:
	
	void draw() 
	
  It draws the picture onto the offscreen image. */
  
public abstract class PlotCanvas extends Canvas implements Corners {
	PlotCTM ctm;
	public Image img;
	public Graphics gfx;
	public int width = 0, height = 0;
	public Applet parent;

	public PlotCanvas(Applet app) {
		// Java 1.1: this(app, app.getBounds().width, app.getBounds().height);
		this(app, app.bounds().width, app.bounds().height);
	}

	public PlotCanvas(Applet app, int w, int h) {
		parent = app;
		setBackground(Color.white);
		width = w;
		height = h;
		ctm = new PlotCTM(width, height);
		img = parent.createImage(width, height);
		gfx = img.getGraphics();
		resize(width, height);
		clear();
	}
	
	public RealRectangle pixel() {
		return(null);
	}

	public void clear() {
		gfx.setColor(getBackground());
		gfx.fillRect(0, 0, width, height);
		gfx.setColor(getForeground());
	}

	public void paint(Graphics g) {
		update(g);
	}

	// If there are extras involved in drawing 
	// this must be overridden

	public void update(Graphics g) {
		// clear the image
		clear();
		// draw the figure into the image
		draw();
		// map the image onto the screen:
		g.drawImage(img, 0, 0, null);
	}
	
	// this routine constructs the picture in the offscreen image
	public abstract void draw();

	public static double interp(double t, double x, double y) {
		return((1.0-t)*x + t*y);
	}
	
	public void setCorners(double llx, double lly, double urx, double ury) {
		ctm.setCorners(llx, lly, urx, ury);
	}
	
	public void setCorners(double llx, double lly, double urx, double ury, 
							int xmargin, int ymargin) {
		double dx = (double) xmargin*(urx - llx)/(double) (width - 2*xmargin);
		double dy = (double) ymargin*(ury - lly)/(double) (height - 2*ymargin);
		ctm.setCorners(llx-dx, lly-dy, urx+dx, ury+dy);
	}
	
	public void translate(double x, double y) {
		ctm.translate(x, y);
	}
	
	public void scale(double x, double y) {
		ctm.scale(x, y);
	}
	
	public void rotate(double x) {
		ctm.rotate(x);
	}
	
	public Point toPoint(double x, double y) {
		return(ctm.toPoint(x, y));
	}

	public Point toPoint(RealPoint P) {
		return(ctm.toPoint(P));
	}

	public RealPoint toRealPoint(Point P) {
		return(ctm.toRealPoint(P));
	}

	public RealPoint toRealPoint(int x, int y) {
		return(ctm.toRealPoint(x, y));
	}
	
	public void setFont(Font f) {
		gfx.setFont(f);
	}
	
	public void drawString(String s, RealPoint Q, Color c) {
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		Point P = ctm.toPoint(Q);
		gfx.drawString(s, P.x, P.y);
		gfx.setColor(tmp);
	}
	
	public void setColor(Color c) {
		gfx.setColor(c);
	}
	
	public void drawString(String s, double x, double y, Color c) {
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		Point P = ctm.toPoint(x, y);
		gfx.drawString(s, P.x, P.y);
		gfx.setColor(tmp);
	}
	
	public void drawString(String s, RealPoint Q) {
		Point P = ctm.toPoint(Q);
		gfx.drawString(s, P.x, P.y);
	}
	
	public void drawString(String s, double x, double y) {
		Point P = toPoint(x, y);
		gfx.drawString(s, P.x, P.y);
	}
	
	public RealRectangle boundary() {
		RealPoint b[] = new RealPoint[4];
		b[0] = toRealPoint(0, height);
		b[1] = toRealPoint(width, height);
		b[2] = toRealPoint(width, 0);
		b[3] = toRealPoint(0, 0);
		RealRectangle r = new RealRectangle(b);
		return(r);
	}	

}
