package real;

/* Copyright 1997 Bill Casselman
   University of British Columbia
   cass@math.ubc.ca */

public class RealPoint {
	public double x, y;

	public RealPoint(double a, double b) {
		x = a;
		y = b;
	}

	public String toString() {
		return("(" + Real.toString(x) + ", " + Real.toString(y) + ")");
	}
	
	public void translate(double a, double b) {
		x += a;
		y += b;
	}
	
	public void scale(double c) {
		x *= c;
		y *= c;
	}
	
	public RealPoint transformedBy(double m[][]) {
		double a, b;
		a = m[0][0]*x + m[0][1]*y;
		b = m[1][0]*x + m[1][1]*y;
		return(new RealPoint(a, b));
	}
	
	// [c -s | s c] [x y] 
	public RealPoint rotated(double theta) {
		double c, s;
		c = Math.cos(theta);
		s = Math.sin(theta);
		RealPoint Q = new RealPoint(c*x - s*y, s*x + c*y);
		return(Q);
	}
	public RealPoint interp(double t, RealPoint P) {
		RealPoint Q = new RealPoint((1.0 - t)*x + t*P.x, (1.0 - t)*y + t*P.y);
		return(Q);
	}
	
	public void normalize() {
		double R = Math.sqrt(x*x + y*y);
		x /= R;
		y /= R;
	}

	public void normalize(double c) {
		double R = Math.sqrt(x*x + y*y);
		c /= R;
		x *= c;
		y *= c;
	}

	public RealPoint normalized(double c) {
		double R = Math.sqrt(x*x + y*y);
		c /= R;
		double X = c*x;
		double Y = c*y;
		return(new RealPoint(X, Y));
	}
	public RealPoint normalized() {
		double R = Math.sqrt(x*x + y*y);
		double X = x/R;
		double Y = y/R;
		return(new RealPoint(X, Y));
	}
}

